package jmaf.core;

import java.io.File;
import java.io.OutputStreamWriter;
import java.util.List;

import jmaf.core.exceptions.CannotCalculateException;


/**
 * Interface must be implemented by rule generators operating
 * on given set of java classes. For external programs it's recommended
 * to extend <code>ExternalRuleGeneratorAbstract</code> class.
 *  
 * @author mjaniszewski
 *
 */
public interface IAlgorithm {
	/**
	 * Denotes normal algorithm termination.	 
	 */
	public static final int STATUS_OK = 0;
	
	/**
	 * Denotes that error or errors occurs during algorithm execution
	 * but it managed to save some part of it's work.
	 */
	public static final int STATUS_OK_WITH_ERRORS = 1;
	
	/**
	 * Denotes abnormal algorithm termination. IT means that created file is corrupted or 
	 * even not created.
	 */
	public static final int STATUS_FATAL_ERROR = 2;
	/**
	 * Denotes user cancelation of computing process. 
	 */
	public static final int STATUS_CANCELED = 3;
	
	/**
	 * Method validating parameter's values. This 
	 * method will be called at least once for each parameter. It should return
	 * error message or <code>null</code> if value is valid.
	 * 
	 * @param parameterID Identifier of the parameter
	 * @param parameterValue Value of the parameter
	 * 
	 * @return <code>null</code> if parameter value is valid, error message otherwise 
	 */
	public String validateParameter(String parameterID, String parameterValue);
	
	/**
	 * Performs algorithm - specific computation on a base of input file and saves it into a output file
	 * 
	 * @param inputFile Input file descriptor
	 * @param outputFile Output file descriptor
	 * @param parameters List of parameter's values with it's ids. List contais two element String arrays
	 * 	containing id and paramters's value, resp.
	 * value of the parameter. Parameters are previously validated using <code>validateParameter()</code> function.
	 * @param outputStream Output stream to put messages on
	 * @param progressMonitor Progress monitor used to inform user about computation's progress. Used only when 
	 * 		appropriate option is set in extension point; <code>null</code> is passed otherwise.
	 * 
	 * @return Algorithm execution status. It should be one of values denoted by following constants:
	 * 	
	 * <ul><li>STATUS_OK
	 * <li>STATUS_OK_WITH_ERRORS
	 * <li>STATUS_FATAL_ERROR
	 * </ul>
	 * @throws CannotCalculateException When calculations cannot be continued. Reason of calculations failure should be described in exception&quot;s message
	 */
	public int performComputation(File inputFile, File outputFile, List<String[]> parameters, OutputStreamWriter outputStream, IProgressNotifier progressMonitor) throws CannotCalculateException;

}

